#-------------------------------------------------------------
#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
# 
#   http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
#
#-------------------------------------------------------------



args <- commandArgs(TRUE)
library(Matrix)

qtle = qexp(as.numeric(args[1])); # default rate = 1.0
p = pexp(qtle);
pl = pexp(qtle, lower.tail=F);

out = matrix(0,nrow=3, ncol=1);
out[1,1] = qtle;
out[2,1] = p;
out[3,1] = pl;

writeMM(as(out, "CsparseMatrix"), args[2]); 

