#-------------------------------------------------------------
#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
# 
#   http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
#
#-------------------------------------------------------------

args <- commandArgs(TRUE)
library(Matrix)

# Rscript ./test/scripts/functions/unary/matrix/QRsolve.R ./test/scripts/functions/unary/matrix/in/A.mtx ./test/scripts/functions/unary/matrix/in/y.mtx ./test/scripts/functions/unary/matrix/expected/x

A = readMM(args[1]); #paste(args[1], "A.mtx", sep=""));
b = readMM(args[2]); #paste(args[1], "b.mtx", sep=""));

m = nrow(A);
n = ncol(A);

Ab = cbind(as.matrix(A), as.matrix(b));

Ab_qr = qr(Ab);
Rb = qr.R(Ab_qr); 

R = Rb[1:n, 1:n];
c = Rb[1:n, (n+1)];

x = solve(R,c);

writeMM(as(x, "CsparseMatrix"), args[3]); #paste(args[2], "x.mtx", sep=""));

