#-------------------------------------------------------------
#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
# 
#   http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
#
#-------------------------------------------------------------


args <- commandArgs(TRUE)
options(digits=22)

library("Matrix")

A <- as.matrix(readMM(paste(args[1], "A.mtx", sep="")))
B <- as.matrix(readMM(paste(args[1], "B.mtx", sep="")))

if( as.integer(args[2])==1 ){
  # MIN
  if( nrow(A)>1 | nrow(B)>1 ){
     if( nrow(B)>nrow(A) ) {
       C <- pmin(B, A);
     }  
     else {
       C <- pmin(A, B);
     }
  }else{
     C <- min(A, B);
  }    
} else{
  # MAX  
  if( nrow(A)>1 | nrow(B)>1 ){
     if( nrow(B)>nrow(A) ){
       C <- pmax(B, A);
     }else{
       C <- pmax(A, B);
     }  
  }else{
     C <- max(A, B);
  } 
}

writeMM(as(as.matrix(C), "CsparseMatrix"), paste(args[3], "C", sep="")); 


