#-------------------------------------------------------------
#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
#
#-------------------------------------------------------------

args<-commandArgs(TRUE)
options(digits=22)
library("Matrix")
library("matrixStats")

# colProds <- function(X) {
#   apply(X, 2, prod)
# }

X = matrix(3, 10, 10)
# X = matrix(seq(9, 100*100+8), 100, 100, byrow=TRUE);
# F = matrix(0.1, 10, 10)
# X = matrix(seq(from = 1, to = 10, by = 2), 10, 20)
# X = matrix(1.3109348721, 10, 10)

R = t(colProds(2*log(X)))
# R = t(colProds(X*min(F)));

writeMM(as(R,"CsparseMatrix"), paste(args[2], "S", sep=""));
