#-------------------------------------------------------------
#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
# 
#   http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
#
#-------------------------------------------------------------


args <- commandArgs(TRUE)
options(digits=22)

library("Matrix")

rlen = as.integer(args[1]);
clen = as.integer(args[2]);

R = matrix(ifelse(as.logical(args[3]), 7, 0), rlen, clen);
if( as.logical(args[4]) ) {
   R[,50:300] = matrix(0, rlen, 251);
}
for(i in 1:10) {
   R = R + matrix(i, rlen, clen);
}

writeMM(as(R, "CsparseMatrix"), paste(args[5], "R", sep=""));
