#!/usr/bin/env bash
#-------------------------------------------------------------
#
# Licensed to the Apache Software Foundation (ASF) under one
# or more contributor license agreements.  See the NOTICE file
# distributed with this work for additional information
# regarding copyright ownership.  The ASF licenses this file
# to you under the Apache License, Version 2.0 (the
# "License"); you may not use this file except in compliance
# with the License.  You may obtain a copy of the License at
# 
#   http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied.  See the License for the
# specific language governing permissions and limitations
# under the License.
#
#-------------------------------------------------------------

mvn package > /dev/null
java -jar -XX:+UseNUMA target/systemds-3.4.0-SNAPSHOT-perf.jar 1008  100  100 1.0 1 30000
java -jar -XX:+UseNUMA target/systemds-3.4.0-SNAPSHOT-perf.jar 1008 1000  100 1.0 1 3000
java -jar -XX:+UseNUMA target/systemds-3.4.0-SNAPSHOT-perf.jar 1008 1000 1000 1.0 1 3000
java -jar -XX:+UseNUMA target/systemds-3.4.0-SNAPSHOT-perf.jar 1008  100  100 0.3 1 30000
java -jar -XX:+UseNUMA target/systemds-3.4.0-SNAPSHOT-perf.jar 1008 1000  100 0.3 1 3000
java -jar -XX:+UseNUMA target/systemds-3.4.0-SNAPSHOT-perf.jar 1008 1000 1000 0.3 1 3000

# java -jar -XX:+UseNUMA target/systemds-3.4.0-SNAPSHOT-perf.jar 1008  100  100 1.0 10 30000
# java -jar -XX:+UseNUMA target/systemds-3.4.0-SNAPSHOT-perf.jar 1008 1000  100 1.0 10 3000
# java -jar -XX:+UseNUMA target/systemds-3.4.0-SNAPSHOT-perf.jar 1008 1000 1000 1.0 10 1000
# java -jar -XX:+UseNUMA target/systemds-3.4.0-SNAPSHOT-perf.jar 1008  100  100 0.3 10 30000
# java -jar -XX:+UseNUMA target/systemds-3.4.0-SNAPSHOT-perf.jar 1008 1000  100 0.3 10 3000
# java -jar -XX:+UseNUMA target/systemds-3.4.0-SNAPSHOT-perf.jar 1008 1000 1000 0.3 10 1000

# java -jar -XX:+UseNUMA target/systemds-3.4.0-SNAPSHOT-perf.jar 1008  100  100 1.0 100 3000
# java -jar -XX:+UseNUMA target/systemds-3.4.0-SNAPSHOT-perf.jar 1008 1000  100 1.0 100 300
# java -jar -XX:+UseNUMA target/systemds-3.4.0-SNAPSHOT-perf.jar 1008 1000 1000 1.0 100 200
# java -jar -XX:+UseNUMA target/systemds-3.4.0-SNAPSHOT-perf.jar 1008  100  100 0.3 100 3000
# java -jar -XX:+UseNUMA target/systemds-3.4.0-SNAPSHOT-perf.jar 1008 1000  100 0.3 100 2000
# java -jar -XX:+UseNUMA target/systemds-3.4.0-SNAPSHOT-perf.jar 1008 1000 1000 0.3 100 1000